package planet.behaviour;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import planet.commonapi.exception.InitializationException;
import planet.util.Utilities;

/**
 * This class allows to add new behaviours to bad nodes on the overlay.
 * Each behaviour is binded to the type of message exchanged between nodes.
 * The message are defined by a type and mode field. It also allow the use
 * of the wildcard '*', that means, whatever type if is specified in type's
 * field or either whatever mode if is specified in mode's field. Through 
 * the method init(), so that their values are initialized. 
 * @author <a href="mailto: marc.sanchez@estudiants.urv.es">Marc Sanchez</a>
 * Date: 10/10/2004
 */
public class BehaviourProperties implements planet.util.BehaviourProperties {
	/**
	 * Behaviour.Property: Specifies The Filter For Behaviours Queue.
	 */
	public static final String FILTER = "FILTER";
	/**
	 * Behaviour.Property: Specifies We Want Debug Or Not Debug Info.
	 */
	public static final String DEBUG_INFO = "DEBUG_INFO";
	/**
	 * Behaviour.Property: Default percentatge of malicious nodes
	 */
	public static final String DEFAULT_FAULTY_NODES = "DEFAULT_FAULTY_NODES";
	/**
	 * Behaviour.Property: Default malicious node distribution
	 */
	public static final String DEFAULT_MALICIOUS_DISTRIBUTION = "DEFAULT_MALICIOUS_DISTRIBUTION";
	/**
	 * Behaviour.Property: Specifies Node's role according to some distribuiton. It allows node's
	 * collusion.
	 */
	public static final String ROLE_SELECTOR = "ROLE_SELECTOR";
	/**
	 * Uniform Distribution.
	 */
	public static final int UNIFORM = 0x1;
	/**
	 * Chain Distribution.
	 */
	public static final int CHAIN = 0x2;
	
	public static final String DISTR[] = { "none",
						     			   "uniform",
						                   "chain"
						                  };
	/*
	 * Percentage of malicious nodes
	 */
	public float percentageOfMalicious;
	/*
	 * Malicious Node distribution
	 */
	public int maliciousDistribution;
	/*
	 * Debug Info flag.
	 */
	public boolean debugInfo = false;
	/*
	 * Role's Selector Class.
	 */
	public Class selectorClass = null;
	/*
	 * Filter Class.
	 */
	public Class filterClass = null;
	/*
	 * Vector will all patterns sorted from more specific to less specific. 
	 */
	public java.util.Vector patterns = new java.util.Vector();
	/*
	 * Behaviour's Set: Good, Bad, Neutral.
	 */
	private java.util.HashSet behSet = new java.util.HashSet();
	/*
	 * When's Set: Local, Remote, Always.
	 */
    private java.util.HashSet whenToSet = new java.util.HashSet();
	/**
	 * Initialize correctly the behaviours.
	 * @throws InitializationException if an error occurs during  the initialization
	 * of the different properties.
	 * @param propertiesFile File with all configuration properties of the concrete 
	 * Behaviours. This file must follow java.util.Properties scheme.
	 */
	public void init(String propertiesFile) throws InitializationException {
		// Load properties
		FileInputStream F = null;
		java.util.Properties P = new java.util.Properties();
		try {
			F = new FileInputStream(propertiesFile);
			P.load(F);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Error On Property File '" + propertiesFile + "'",e);
		} catch (IOException e) {
			throw new InitializationException("Error On Property File '" + propertiesFile + "'",e);
		}
		
		// Init Properties sets
		setBehSet();
		setWhenToSet();
		
		// Filter
		if (P.containsKey(FILTER)) {
			try {
				filterClass = Class.forName(P.getProperty(FILTER));
				Class[] interfaces = filterClass.getInterfaces();
				boolean filterInterface = false;
				for(int it = 0; it < interfaces.length; it++) 
					if (interfaces[it].getName().equals("planet.behaviour.Filter")) filterInterface = true;
				if (!filterInterface) throw new InitializationException("Filter's Class '" + filterClass.getName() + "' Does Not Implement Interface Filter");
				P.remove(FILTER);
			} catch(ClassNotFoundException e) {
				throw new InitializationException("Filter's Class '" + P.getProperty(FILTER) + "' Has Not Found It.", e);
			}
		}
		// Debug Info
		if (P.containsKey(DEBUG_INFO)) {
			debugInfo = Utilities.parseBoolean(P.getProperty(DEBUG_INFO));
			P.remove(DEBUG_INFO);
		}
		
		// ROLE_SELECTOR
		if (P.containsKey(ROLE_SELECTOR)) {
			try {
				selectorClass = Class.forName(P.getProperty(ROLE_SELECTOR));
				Class[] interfaces = selectorClass.getInterfaces();
				boolean selectorInterface = false;
				for (int it = 0; it < interfaces.length; it++)
					if (interfaces[it].getName().equals("planet.behaviour.BehaviourRoleSelector")) selectorInterface = true;
				if (!selectorInterface) throw new InitializationException("Role Selector's Class '" + selectorClass.getName() + 
						"'Does Not Implement Role's Selector Interface");
				
			} catch(ClassNotFoundException e) {
				throw new InitializationException("Role's Selector Class '" + P.getProperty(ROLE_SELECTOR) + "' Has Not Found It.", e);
			}
		}
		
		// DEFAULT_FAULTY_NODES
		if (P.containsKey(DEFAULT_FAULTY_NODES)) {
			int percentage = Integer.parseInt(P.getProperty(DEFAULT_FAULTY_NODES));
			if (percentage < 0 || percentage > 100) 
					throw new InitializationException("Error, '"+  P.getProperty(DEFAULT_FAULTY_NODES) + "' Is Not a Valid Percentatge Of Faulty Nodes.");
			percentageOfMalicious = (float) percentage / 100;
			P.remove(DEFAULT_FAULTY_NODES);
		} else if (P.containsKey(ROLE_SELECTOR)) {
			throw new InitializationException("Percentatge Of Faulty Nodes Is Missing.");
		}
		
		// DEFAULT_MALICIOUS_DISTRTIBUTION
		if (P.containsKey(DEFAULT_MALICIOUS_DISTRIBUTION)) {
				String distribution = P.getProperty(DEFAULT_MALICIOUS_DISTRIBUTION);
				if (distribution.equals("UNIFORM")) maliciousDistribution = UNIFORM;
				else if (distribution.equals("CHAIN")) maliciousDistribution = CHAIN; 
				else throw new InitializationException("Error, '" + P.getProperty(DEFAULT_MALICIOUS_DISTRIBUTION) + "' Is Not A Valid Distribution Type.");
				P.remove(DEFAULT_MALICIOUS_DISTRIBUTION);
		} else if (P.containsKey(ROLE_SELECTOR)) {
			throw new InitializationException("Malicious Distribution Property Is Missing.");
		}
		
		P.remove(ROLE_SELECTOR);

		// Behaviour's List
		String key = null;
		Class behaviourClass = null;
		java.util.Enumeration S = P.propertyNames();
		while(S.hasMoreElements()) {
			 key = (String) S.nextElement();
  			 try {
					behaviourClass = Class.forName(key);
					Class[] interfaces = behaviourClass.getInterfaces();
					boolean behInterface = false;
					for(int it = 0; it < interfaces.length; it++) 
						if (interfaces[it].getName().equals("planet.behaviour.Behaviour")) behInterface = true;
					if (!behInterface) throw new InitializationException("Behaviour's Class '" + behaviourClass.getName() + "' Does Not Implement Interface Behaviour");
			} catch (ClassNotFoundException e) {
				throw new InitializationException("Behaviour's Class '" + key + "' Has Not Found It.", e);
			}
			java.util.StringTokenizer parser = new java.util.StringTokenizer(P.getProperty(key), ",");
			String typeOf = null, modeOf = null, whenTo = null, roleOf = null;
			double pdf;
			try {
				typeOf = (String) parser.nextElement();
				modeOf = (String) parser.nextElement();
				if (typeOf.equals(Pattern.Cwildcard) && modeOf.equals(Pattern.Cwildcard))
						throw new InitializationException("Error, Pattern '?, ?' Invalid, You Might Refer to '*, ?'");
			    pdf = Double.parseDouble((String) parser.nextElement());
			    if (!isNormalized(pdf)) throw new InitializationException("Error, Probability Must Be On (0.0, 1.0]");
			    whenTo = ((String) parser.nextElement()).trim();
			    if (!lexWhenTo(whenTo)) throw new InitializationException("Error, Unknown Token '" + whenTo + "' For When Property");
			    roleOf = ((String) parser.nextElement()).trim();
			    if (!lexBehSet(roleOf)) throw new InitializationException("Error, Unknown Token '" + roleOf + "' For Role Property");
		    	this.add(new Pattern(behaviourClass, typeOf.trim(), modeOf.trim(), pdf, whenTo, roleOf));
			} catch(java.util.NoSuchElementException e1) {
				throw new InitializationException("Error, Some Field Must Be Omitted", e1);
			} catch (NumberFormatException e2) {
				throw new InitializationException("Error, Frequency Field Might Be Bad Formatted", e2);
			}
		}
	}
	/**
	 * @return Returns a vector with all pattern sorted from more-to-less specific.
	 */
	public java.util.Vector getPatterns() {
		return patterns;
	}
	/**
	 * @return Returns the filter class
	 */
	public Class getFilter() {
		return filterClass;
	}
	/**
	 * This method add patterns to the vector sorted from more-to-less specific.
	 * Specific criteria:
	 * 		   Type  Mode	
	 * 		1) Tag,  Tag
	 * 		2) Tag,   ?
	 * 		3) Tag,   *
	 * 		4)  ? ,  Tag
	 *      5)  * ,  Tag
	 *      6)  ? ,   *
	 *      7)  * ,   ?
	 *      8)  * ,   *
	 * @param pattern Pattern to insert.
	 */
	protected void add(Pattern pattern) {
		int at = 0; 
		for (at = 0; at < patterns.size(); at++) {
			Pattern p = (Pattern) patterns.get(at);
			if (p.compareTo(pattern) < 0)  break;
		}
		patterns.add(at, pattern);
	}
	/**
	 * This method checks wether a double value lies between (0.0, 1.0] range.
	 * @param d double value to check
	 * @return true if d lies between (0.0, 1.0].
	 */
	protected boolean isNormalized(double d) {
		return d > 0.0 && d <= 1.0;
	}
	 /**
     * Set's Behaviour Set with Behaviour properties: Pattern.Good, Pattern.Bad, 
     * Pattern.Neutral. @see planet.behvaiour.Pattern
     */
    private void setBehSet() {
		behSet.add(Pattern.Good);		// Good's Node Behaviour
		behSet.add(Pattern.Bad);		// Bad's Node Behaviour
		behSet.add(Pattern.Neutral);	// Neutral's Node Behaviour
	}
    /**
     * Set's When a Behaviour should get RUN state: Pattern.Local, Pattern.Remote,
     * Pattern.Always. @see planet.behaviour.Pattern  
     */
	private void setWhenToSet() {
		whenToSet.add(Pattern.Local);		// For the Local Node
		whenToSet.add(Pattern.Remote);		// For a Remote Node
		whenToSet.add(Pattern.Always);		// Always
	}
	/**
	 * Returns true if Node's behaviours is a token of the grammar.
	 * @param beh The token to be checked.
	 * @return True if beh belongs to language.
	 */
	private boolean lexBehSet(String beh) {
		return behSet.contains(beh);
	}
	/**
	 * Returns true if When's property is a token of the grammar.
	 * @param whenTo The token to be checked.
	 * @return True if whenTo belongs to language.
	 */
	private boolean lexWhenTo(String whenTo) {
		return whenToSet.contains(whenTo);
	}
	/**
	 * Prints sorted patterns from more-to-less specific.
	 */
	protected void printPatterns() {
		java.util.Iterator it = patterns.iterator();
		while(it.hasNext()) System.out.println(it.next());
	}
}
