package planet.behaviour;
/**
 *	This class stores a pattern for incoming messages on a node. This pattern is used to
 *  dispatch a behaviour <b>classOf</b> whenever the pattern matches the type and mode of 
 *  the incoming message. To allow behaviours to execute randomly on time, every pattern
 *  has a probability.
 *  @author <a href="mailto: marc.sanchez@estudiants.urv.es">Marc Sanchez</a>
 *  Date: 10/10/2004   
 */
public class Pattern {
	/**
	 * Complementary wildcard: given a set of patterns P, this wildcard is used 
	 * to attach a behaviour to the patterns on the whole pattern space S not
	 * included in P set. 
	 */
	public static final String Cwildcard = "?";
	/**
	 * Universal wildcard: given a set of patterns P, this wildcard is used 
	 * to attach a behaviour to all the patterns on the whole pattern space 
	 * S included P set, i.e. P C S. 
	 */
	public static final String Uwildcard = "*";
	/*
	 * The number of possible tokens: Complementary wildcard "?", Universal
	 * wildcard "*" and tag token.
	 */
	private static final int tokens = 3;
	/**
	 * Overlay property: Local refers to a behaviour which must run only
	 * when RouteMessage's pattern is for him.
	 */
	public static final String Local   = "LOCAL";
	/**
	 * Overlay property: Remote refers to a behaviour which must run only
	 * when RouteMessage's pattern is for another peer rather than him.
	 */
	public static final String Remote  = "REMOTE";
	/**
	 * Overlay property: Always refers to a behvaiour which must run always
	 * despite RouteMessage's destination. 
	 */
	public static final String Always  = "ALWAYS";
	/**
	 * Overlay property: Good refers to a behaviour which must run when node
	 * behaves as underlying kbr protocol stipulates.
	 */
	public static final String Good    = "GOOD";
	/**
	 * Overlay property: Bad refers to a behvaiour which must run when node
	 * misbehaves, including routing and overlay invariant's maintenance.
	 */
	public static final String Bad     = "BAD";
	/**
	 * Overlay property: Neutral refers to a behvaiour which must run always
	 * despite node's role is bad or good.
	 */
	public static final String Neutral = "NEUTRAL";
    /*
     * Maps a pattern to an integer.
     */
	private int map;
	/*
	 * Chance for behaviour <b>classOf</b> to get scheduled.  
	 */
	private double pdf;
	/*
	 * Pattern Matching: Type's field of incoming messages.
	 */
	private String typeOf;
	/*
	 * Pattern Matching: Mode's field of incoming messages.
	 */
	private String modeOf;
	/*
	 * Instances of the class Class represent classes and interfaces in a running Java application.
	 */
	private Class  classOf;
	/*
	 * Specifies when to execute some behaviour.
	 */
	private String whenTo;
	/*
	 * Specifies the role of a behaviour.
	 */
	private String roleOf;
	 
	// Constructor
	public Pattern(Class classOf, String typeOf, String modeOf, double pdf, String whenTo, String roleOf) {
		this.classOf = classOf;
		this.typeOf = typeOf;
		this.modeOf = modeOf;
		this.pdf = pdf;
		this.whenTo = whenTo;
		this.roleOf = roleOf;
		this.map = map(typeOf)* tokens + map(modeOf);
	}
	/**
	 * @return Returns the class of behaviour.
	 */
	public Class getClassOf() {
		return classOf;
	}
	/**
	 * @return Returns the mode of pattern.
	 */
	public String getModeOf() {
		return modeOf;
	}
	/**
	 * @return Returns the probability binding.
	 */
	public double getPdf() {
		return pdf;
	}
	/**
	 * @return Returns the type of the pattern.
	 */
	public String getTypeOf() {
		return typeOf;
	}
	/**
	 * @return Returns when to exec this behaviour.
	 */
	public String getWhenTo() {
		return this.whenTo;
	}
	/**
	 * @return Returns the role of this behaviour.
	 */
	public String getRoleOf() {
		return this.roleOf;
	}
	/**
	 * @return Returns an integer mapping for this pattern.
	 */
	public int getMap() {
		return map;
	}
	/**
	 * Maps a property to specific quantifier. So Complementary wildcard <b>"?"</b> is more 
	 * specific than universal wildcard <b>"*"</b>. 
	 * @param property The property to quantify.
	 * @return The quantifier of the property. 
	 */
	public int map(String  property) {
		int map = -tokens; 
		if (property.equals(Cwildcard)) map = 1;
		else if (property.equals(Uwildcard)) map = 2;
		return map;  
	}
	/**
	 * Compares two patterns based on specific criteria.
	 *   Specific criteria:
	 * 		   Type  Mode	
	 * 		1) Tag,  Tag
	 * 		2) Tag,   ?
	 * 		3) Tag,   *
	 * 		4)  ? ,  Tag
	 *      5)  * ,  Tag
	 *      6)  ? ,   *
	 *      7)  * ,   ?
	 *      8)  * ,   *
	 * @param pattern The pattern to compare.
	 * @return the value 0 if the argument pattern is equal to this pattern; 
	 * 	a value less than 0 if this pattern is less specific than the pattern argument; 
	 *  and a value greater than 0 if the pattern is more specific than the pattern argument.
	 */
	public int compareTo(Pattern pattern) {
		if (pattern.getMap() == this.map) return 0;
		else if (pattern.getMap() < this.map) return -1;
		return 1;
	}
	
	public String toString() {
		return "Behaviour_________________________\n\n\t" + classOf.getName() 
		+ "\n\tPattern [[ " + typeOf + " AND " + modeOf + "]]" 
		+ "\n\tPdf     " + pdf
		+ "\n\tWhen	   " + whenTo
		+ "\n\tRole	   " + roleOf
		+ "\n\tMap     " + map
		+ "\n__________________________________";
	}
}
