package planet.generic.commonapi.factory;

import java.io.*;
import java.lang.reflect.Constructor;

import planet.commonapi.*;
import planet.commonapi.exception.InitializationException;
import planet.commonapi.factory.*;
import planet.util.Properties;

/**
 * This factory abstracts a NetworkFactory. The file passed by parameter at constructor
 * must contains: 
 * <ul>
 * <li>With any class of Network implementation</li>
 * <li>With any type of Node.</li>
 * <li>With any defined topology</li>
 * <li>With any total size of the network (bits for its Id)</li>
 * <li>Within any number of nodes</li>
 * </ul>
 * The existing properties in that file are the follows:
 * <ol>
 * <li><b>DEFAULT_NETWORK</B>: An implementation of Network interface.</LI>
 * <li><b>DEFAULT_NODEFACTORY</b>: A NodeFactory implementation to
 * use for build nodes within the network.</li>
 * <li><b>DEFAULT_SIZE</b>: Number of nodes to generate under the network.</li>
 * <li><b>DEFAULT_K</b>: Number of bits for the Ids for the Nodes.</li>
 * <li><b>DEFAULT_SERIALIZEDFILE</b>: The file where appear the serialized state
 * of some network.</b>
 * <li><b>DEFAULT_TOPOLOGY</b>: Topology of the network to construct.</li>
 * </ol>
 * All those properties can be modified too by invoking concrete methods
 * within the Network.
 * @author Jordi Pujol
 * @see planet.commonapi.Network
 */
public class NetworkFactoryImpl implements NetworkFactory {
	/**
	 * The Constructor for Networks.
	 */
	protected Constructor networkConstructor = null;
	/**
	 * The NodeFactory actually in use.
	 */
	protected NodeFactory nodeFactory = null;
	/**
	 * The default network size to use for build a network.
	 */
	protected int networkSize;
	/**
	 * Number of bits for the Ids of Nodes.
	 */
	protected int bitsKey;
	/**
	 * The default topology to use for build a network.
	 */
	protected String topology = null;
	/**
	 * The default filename from where read a serialized state of
	 * a network.
	 */
	protected String serializedFile = null;
	
	/**
	 * Builds a new instance of NetworkFactory reading all necessary properties from
	 * <b>propertiesFile</b>. 
	 * @param propertiesFile File of properties which contains all properties of factories.
	 * @throws InitializationException if occur any problem during initialization.
	 */
	public NetworkFactoryImpl(String propertiesFile) throws InitializationException {
		//Load properties
		FileInputStream fis = null;
		java.util.Properties netProp = new java.util.Properties();
		String propertiesFault = "";
		try {
			fis = new FileInputStream(propertiesFile);
			netProp.load(fis);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Cannot read the file "+propertiesFile,e);
		} catch (IOException e) {
			throw new InitializationException("Cannot load the file "+propertiesFile,e);
		}
		
		//if all OK
		//read defaultNetwork class
		Class networkClass = null;
		try {
			networkClass = Class.forName(netProp.getProperty(Properties.DEFAULT_NETWORK));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+Properties.DEFAULT_NETWORK +"] is not a valid class name.",e);
		}
		
		//read Properties.DEFAULT_NODEFACTORY class
		Class nodeFactClass = null;
		try {
			nodeFactClass = Class.forName(netProp.getProperty(Properties.DEFAULT_NODE_FACTORY));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+Properties.DEFAULT_NODE_FACTORY +"] is not a valid class name.",e);
		}
		//read Properties.DEFAULT_TOPOLOGY 
		topology = netProp.getProperty(Properties.DEFAULT_TOPOLOGY);
		if (!Topology.isValid(topology)) 
			throw new InitializationException("Property ["+Properties.DEFAULT_TOPOLOGY +"] is not a valid topology.");

		//read Properties.DEFAULT_NETWORK_SIZE 
		networkSize = Integer.parseInt(netProp.getProperty(Properties.DEFAULT_NETWORK_SIZE));
		
		//Class nodeFactClass, Class idFactory, Class nodeClass, Class idClass, String topology, int networkSize
		init(propertiesFile,networkClass,nodeFactClass);
	}
	
	/**
	 * Builds a new instance of the NetworkFactory using all specified parameters. All ones are not
	 * nessary for NetworkFactory, but are necessary for build required Factories.
	 * @param nodeFactClass NodeFactory class to use to build new nodes.
	 * @param idFactory IdFactory class to use to build new Ids.
	 * @param nodeClass Node class to use to build new nodes.
	 * @param idClass Id class to use to build new Ids.
	 * @param networkSize Size of the target network to build. 
	 * @param topology Topology of the target network to build.
	 * @throws InitializationException if occurs any problem during initialization.
	 */
	public NetworkFactoryImpl(Class networkClass, Class nodeFactClass, Class idFactory, Class nodeClass, Class idClass, String topology, int networkSize) throws InitializationException {
		init(networkClass,nodeFactClass, idFactory, nodeClass, idClass, topology, networkSize);
	}
	
	/**
	 * Initialize this NetworkFactory with the specified parameters. All parameters are not necessary for
	 * NetworkFactory, but are necessary for build required Factories.
	 * @param nodeFactClass NodeFactory class to use to build new nodes.
	 * @param idFactory IdFactory class to use to build new Ids.
	 * @param nodeClass Node class to use to build new nodes.
	 * @param idClass Id class to use to build new Ids.
	 * @param networkSize Size of the target network to build. 
	 * @param topology Topology of the target network to build.
	 * @throws InitializationException if occurs any problem during initialization.
	 */
	private void init(Class networkClass, Class nodeFactClass, Class idFactory, Class nodeClass, Class idClass, String topology, int networkSize) throws InitializationException {
		
		
		//read Properties.DEFAULT_NODEFACTORY
		try {
			//Class idFactory, Class nodeClass, Class idClass, String topology,int networkSize
			Class[] types = {Class.class, Class.class, Class.class, String.class, int.class};
			Constructor construct = nodeFactClass.getConstructor(types);
			Object[] params = {idFactory,nodeClass,idClass,topology,new Integer(networkSize)};
			nodeFactory = (NodeFactory) construct.newInstance(params);
		} catch (Exception e) {
			throw new InitializationException("Cannot generate an instance of IdFactory.",e);
		}
		
		//read defaultNetwork
		try {
			//Class nodeFactClass
			Class[] types = {NodeFactory.class};
			networkConstructor = networkClass.getConstructor(types);
		} catch (Exception e) {
			throw new InitializationException("Cannot generate the Constructor for Network.",e);
		}
		
		this.networkSize = networkSize;
		this.topology = topology;
	}
	
	/**
	 * Initialize this instance of NodeFactory reading directly of the properties file to generate
	 * the required factories.
	 * @deprecated This method just reads directly from file of properties. We recomended not use this method.
	 * It just exist for coherence.
	 * @param nodeFactClass NodeFactory class to be generated.
	 * @param networkSize Size of the target network.
	 * @param topology Topology of the target network.
	 * @throws InitializationException if occurs any problem during initialization.
	 */
	private void init(String propertiesFile, Class networkClass, Class nodeFactClass) throws InitializationException {
		//read Properties.DEFAULT_NODEFACTORY
		try {
			Class[] types = {String.class};
			Constructor construct = nodeFactClass.getConstructor(types);
			Object[] params = {propertiesFile};
			nodeFactory = (NodeFactory) construct.newInstance(params);
		} catch (Exception e) {
			throw new InitializationException("Cannot generate an instance of IdFactory.",e);
		}

		//read defaultNetwork
		try {
			//Class nodeFactClass
			Class[] types = {NodeFactory.class};
			networkConstructor = networkClass.getConstructor(types);
		} catch (Exception e) {
			throw new InitializationException("Cannot generate the Constructor for Network.",e);
		}

	}
	
	/**
	 * Builds a network with the actual specified properties. It is,
	 * NodeFactory, size of network, number of bits and topology.
	 * @see planet.commonapi.factory.NetworkFactory#buildNetwork()
	 * @return A network with the specified number of Nodes.
	 * @throws InitializationException if occurs any problem during
	 * the process to build the network.
	 */
	public Network buildNetwork() throws InitializationException {
		return _buildNetwork(this.networkSize,this.nodeFactory,this.topology,this.serializedFile,this.bitsKey);
	}
	
	/**
	 * Builds a network with the actual specified properties, but
	 * overwriting the number of nodes to build within.
	 * @see planet.commonapi.factory.NetworkFactory#buildNetwork(int)
	 * @param size The number of Nodes for the new Network
	 * @return A network with the specified <b>size</b> of Nodes.
	 * @throws InitializationException if occurs any problem during
	 * the process to build the network.
	 */
	public Network buildNetwork(int size) throws InitializationException {
		return _buildNetwork(size,this.nodeFactory,this.topology,this.serializedFile,this.bitsKey);
	}
	
	/**
	 * Builds a network with the specified <b>size</b> for the network, building it
	 * under this concrete <b>topology</b>. The rest of properties are used the
	 * actually ones in use.
	 * @see planet.commonapi.factory.NetworkFactory#buildNetwork(int, java.lang.String)
	 * @param size The number of Nodes for the new Network
	 * @param topology The network topology to constructs new network.
	 * @return A network with the specified <b>size</b> of Nodes.
	 * @throws InitializationException if occurs any problem during
	 * the process to build the network.
	 */
	public Network buildNetwork(int size, String topology) throws InitializationException {
		return _buildNetwork(size,this.nodeFactory,topology,this.serializedFile,this.bitsKey);
	}

	/**
	 * Builds a new network with the specified <b>size</b> and this concrete
	 * <b>nodeFactory</b>. The rest of properties are used the actually ones
	 * in use.
	 * @see planet.commonapi.factory.NetworkFactory#buildNetwork(int, planet.commonapi.factory.NodeFactory)
	 * @param size The number of Nodes for the new Network
	 * @param nodeFactory The NodeFactory to use to build Nodes for the new network.
	 * @return A network with the specified <b>size</b> of Nodes.
	 * @throws InitializationException if occurs any problem during
	 * the process to build the network.
	 */
	public Network buildNetwork(int size, NodeFactory nodeFactory) throws InitializationException {
		return _buildNetwork(size,nodeFactory,this.topology,this.serializedFile,this.bitsKey);
	}

	/**
	 * Builds a new network with the specified <b>size</b>, this concrete
	 * <b>nodeFactory</b> and network <b>topology</b>. 
	 * The rest of properties are used the actually ones in use.
	 * @see planet.commonapi.factory.NetworkFactory#buildNetwork(int, planet.commonapi.factory.NodeFactory, java.lang.String)
	 * @param size The number of Nodes for the new Network
	 * @param nodeFactory The NodeFactory to use to build Nodes for the new network.
	 * @param topology The network topology to constructs new network.
	 * @return A network with the specified <b>size</b> of Nodes.
	 * @throws InitializationException if occurs any problem during
	 * the process to build the network.
	 */
	public Network buildNetwork(int size, NodeFactory nodeFactory,
			String topology) throws InitializationException {
		return _buildNetwork(size,nodeFactory,topology,this.serializedFile,this.bitsKey);
	}

	/**
	 * Builds a new network with the specified <b>size</b>, this concrete
	 * <b>nodeFactory</b>, network <b>topology</b> and <b>k</b> bits to
	 * construct new Ids for the Nodes. In this case, the default properties
	 * are not used.
	 * @see planet.commonapi.factory.NetworkFactory#buildNetwork(int, planet.commonapi.factory.NodeFactory, java.lang.String, int)
	 * @param size The number of Nodes for the new Network
	 * @param nodeFactory The NodeFactory to use to build Nodes for the new network.
	 * @param topology The network topology to constructs new network.
	 * @return A network with the specified <b>size</b> of Nodes.
	 * @throws InitializationException if occurs any problem during
	 * the process to build the network.
	 */
	public Network buildNetwork(int size, NodeFactory nodeFactory,
			String topology, int k) throws InitializationException {
		return _buildNetwork(size,nodeFactory,topology,this.serializedFile,k);
	}
	
	/**
	 * Builds a network with the specified parameters. If the <b>topology</b> is Topology.SERIALIZED,
	 * it load the network state from the <b>serializedState</b> path.
	 * @param size Number of nodes to build under in the network.
	 * @param nodeFactory NodeFactory to use to build new Nodes.
	 * @param topology Topology of the new network.
	 * @param serializedState Filename which contains a serialized state of some network.
	 * @param k Number of bits of Nodes Ids.
	 * @return An instance of a Network. This network is a deserialized Network if <b>topology</b>
	 * is a Topology.SERIALIZED, o a new instance in other case, with those parameters.
	 * @throws InitializationException if occurs any problem with the process to obtain the
	 * network instance.
	 */
	private Network _buildNetwork(int size, NodeFactory nodeFactory, String topology, 
		String serializedState, int k) throws InitializationException {
		if (topology.equals(Topology.SERIALIZED)) {
			Network netToReturn = null;
			try {
				ObjectInputStream ois = new ObjectInputStream(new FileInputStream(serializedState));
				netToReturn = (Network)ois.readObject();
				ois.close();
			} catch (FileNotFoundException e) {
				throw new InitializationException("Cannot find file ["+serializedState+"].",e);
			} catch (Exception e) {
				throw new InitializationException("Cannot load the serialized state of network from the file ["+serializedState+"].",e);
			}
			return netToReturn;
		}
		Object[] params = {nodeFactory};
		Network net;
		try {
			net = (Network)networkConstructor.newInstance(params);
		} catch (Exception e) {
			throw new InitializationException("Cannot build the network.",e);
		} 
		net.joinNodes(size);
		return net;
	}
		
	/**
	 * Returns the NodeFactory actually in use.
	 * @see planet.commonapi.factory.NetworkFactory#getNodeFactory()
	 * @return The NodeFactory actually in use.
	 */
	public NodeFactory getNodeFactory() {
		return nodeFactory;
	}
	
	/**
	 * Sets the NodeFactory to use in the future. 
	 * @see planet.commonapi.factory.NetworkFactory#setNodeFactory(planet.commonapi.factory.NodeFactory)
	 * @param nodeFactory NodeFactory to use in the future.
	 */
	public void setNodeFactory(NodeFactory nodeFactory) {
		this.nodeFactory = nodeFactory;
	}
	
	/**
	 * Inform from the network size for the default building networks.
	 * @see planet.commonapi.factory.NetworkFactory#getNetworkSize()
	 * @return The number of Nodes for the default building networks.
	 */
	public int getNetworkSize() {
		return networkSize;
	}
	
	/**
	 * Sets a new size for the default building networks.
	 * @see planet.commonapi.factory.NetworkFactory#setNetworkSize(int)
	 * @param size The new size for the default building networks.
	 */
	public void setNetworkSize(int size) {
		this.networkSize = size;
	}
	
	/**
	 * Inform the topology actually in use to build networks.
	 * @see planet.commonapi.factory.NetworkFactory#getTopology()
	 * @return The topology in use.
	 */
	public String getTopology() {
		return topology;
	}
	
	/**
	 * Sets the topology for the networks to build in the future.
	 * @see planet.commonapi.factory.NetworkFactory#setTopology(java.lang.String)
	 * @param topology The new topology for the default building networks.
	 */
	public void setTopology(String topology) throws InitializationException {
		this.topology = topology;
	}
	
	
	/**
	 * Gets the filename for the serialized state of some network.
	 * @see planet.commonapi.factory.NetworkFactory#getSerializedFile()
	 * @return The filename that contains a serialized state of some network.
	 */
	public String getSerializedFile() {
		return serializedFile;
	}

	/**
	 * Sets the filename for capture a serialized state of some network.
	 * @see planet.commonapi.factory.NetworkFactory#setSerializedFile(java.lang.String)
	 * @param file Filename from where read the serialized state of a network.
	 * @throws InitializationException if occur some problem with that <b>file</b>
	 */
	public void setSerializedFile(String file) throws InitializationException {
		this.serializedFile = file;
	}
}
