package planet.simulate;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Random;
import java.util.Set;
import java.util.Stack;
import java.util.Vector;

import planet.chord.ChordProperties;
import planet.commonapi.Id;
import planet.commonapi.Node;
import planet.commonapi.NodeHandle;
import planet.commonapi.RouteMessage;
import planet.commonapi.exception.InitializationException;
import planet.commonapi.factory.IdFactory;
import planet.commonapi.factory.NodeFactory;
import planet.generic.commonapi.NodeImpl;
import planet.generic.commonapi.factory.GenericFactory;
import planet.util.Properties;
import planet.util.Queue;
import planet.util.QueueFull;

/**
 *  The simulator is the main class.  It controls the events to execute them 
 *  at the opportune moment, controls the nodes, while they join, go away, they fail,
 *  and handles to the messages and the message queues
 * @author Pedro Garca
 * @author Carles Pairot
 * @author Ruben Mondejar 
 */
public class Simulator implements ISimulator, java.io.Serializable {
  private int time = 0;
  private HashMap nodes;
  private IScheduler timer;
  private Stack toRemove;
  private int stab = 0;
  private NodeFactory nodeFactory = null;
  private IdFactory idFactory = null;
  
  /**
   * Constructor, create a new simulation with a determinate event scheduler
   * @param timer Scheduler with the all events to simulate
   * @see planet.simulate.IScheduler
   */       
  public Simulator (IScheduler timer) throws InitializationException {
  	MessagePool.init();
  	Properties.init("../conf/factories.properties","../conf/sim.properties","../conf/chord.properties");
  	nodeFactory = GenericFactory.getNodeFactory();
  	idFactory = GenericFactory.getIdFactory();
    nodes = new HashMap();
    toRemove = new Stack();
    this.timer = timer;    
	Results.clearStabRate();
  }
  
  /**
   * Constructor, create a new simulation with a determinate event scheduler and with indicated argument,
   * will load a old state of simulation
   * @param timer Scheduler with the all events to simulate
   * @param load if true, load a old state
   * @see planet.simulate.IScheduler
   */  
  public Simulator (IScheduler timer, boolean load) throws IOException, FileNotFoundException, ClassNotFoundException, InitializationException {
      MessagePool.init();
  	  Properties.init("../conf/factories.properties","../conf/sim.properties","../conf/chord.properties");

  	  if (load) {  	  
	    //ObjectInputStream fis = new ObjectInputStream (new FileInputStream (Globals.INFILE_NAME));
  	  	ObjectInputStream fis = new ObjectInputStream (new FileInputStream (Properties.serializedFile));
	    nodes = (HashMap)fis.readObject();
  	  }
  	  else nodes = new HashMap();
	  toRemove = new Stack();
	  this.timer = timer;    
	  Results.clearStabRate();
  }

  /**
   *  Executes the events at the opportune moment, controls the nodes, 
   *  while they join, go away, they fail,and handles to the messages 
   *  and the message queues
   *  @param steps number of steps that the simulation lasts 
   */  
  public void run (int steps) throws InitializationException {
  	for (int i=0; i< steps; i++)
  		simulate();
  }
  
  /**
   * Serialize the actual state.
   */
  public void saveState() {
	try {
	 ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(Properties.outputFile));
	 os.writeObject (nodes);
	 os.close();
	}
	catch (Exception ex) {
	  ex.printStackTrace();
	} 
  }
  /**
   *  Executes step by step the events at the opportune moment, controls the nodes, 
   *  while they join, go away, they fail,and handles to the messages 
   *  and the message queues. It stops when the events over and the
   *  stabilization is been completed.
   *  @return finish return true if the simulation is been finished 
   */  
  
  public boolean simulate() throws InitializationException {
  	Results.clearStabRate();
  	
	Vector events = new Vector();
	Collection nodeList;
	Iterator it,it2;
	Node aNode;
	Queue messages;	
	int max_messages = 0;

	events = timer.getEvents(time);
	if (!events.isEmpty())
	  parseEvents(events);

	nodeList = nodes.values();
	it = nodeList.iterator();
	// Each node process incoming messages and handles additional stabilization operations
	while (it.hasNext()){
	  aNode = (Node)it.next();
	  aNode.process(this.time);
	}
	
	it2 = nodeList.iterator();
	// The simulator moves outgoing messages to target's incoming node queues
	while (it2.hasNext()){
	  aNode = (Node)it2.next();
	  messages = aNode.outMessages();
	  if (messages.size()>max_messages) max_messages = messages.size();	  
	  //Results.numMessagesTime(aNode.getId(),time,messages.size(),"out");
	  send(messages);
	}
	removeNodes();
	
	time++; 	
	
	if (time%1000==0) {
	  Logger.log("Simulator: Simulation time "+time+" and num nodes "+nodes.size()+" num messages "+max_messages,Logger.EVENT_LOG);	          
	  //if (Results.getStabRate()==0) Logger.log("ESTABILITZAT!!!!!",Globals.EVENT_LOG);
	  //else Logger.log("Ratio estabilitzacio "+Results.getStabRate(),Globals.EVENT_LOG);	
	  Logger.log("Simulator: Ratio estabilitzacio "+Results.getStabRate(),Logger.EVENT_LOG);
	}     
		 
	Logger.setStep(time);
	//if (oldRate == Results.getStabRate()) stab++;
	if (Results.getStabRate()==0) stab++;
	else stab = 0;	 	   
  	
	//return (timer.hasNext() || stab<Globals.T_FixFingers*Globals.K || Results.getTraffic()>0);
	//translation
	return (timer.hasNext() || 
			stab < (((ChordProperties)Properties.overlayProps).stabilizeSteps*Properties.bitsKey) || 
			Results.getTraffic() > 0);
  }

  /**
   * Joins a new node to the simulated network.
   *
   * @param node New node to add to the network.
   * @param bootstrap Id of the any node in the network
   */ 
  public void addNode(planet.commonapi.Node node, Id bootstrap) {    
	nodes.put(node.getId(),node);
	node.join(bootstrap);
	
  }

  public void printNodes() {
  	NodeImpl aNode = null;
  	Collection c = nodes.values();
    Iterator it3 = c.iterator();
    while (it3.hasNext()){
      aNode = (NodeImpl)it3.next();
      aNode.printNode();
    }

    Set li = nodes.keySet();
    
    Results.print();
  }
  
  /**
   * Create and joins a new node to the simulated network.
   *
   * @param id Id of the new node
   * @param bootstrap Id of the any node in the network
   * @return node a reference to new node
   */ 
  public planet.commonapi.Node addNode(Id id, Id bootstrap) throws InitializationException {
	Node newNode = nodeFactory.buildNode(id);
	nodes.put(id,newNode);
	newNode.join(bootstrap);
	
	return (planet.commonapi.Node) newNode;	
  }
  
  /**
   * Removes the ready nodes to delete
   */ 
  public void removeNodes() {
    Id mid;
    while (!toRemove.isEmpty()){
      mid = (Id)toRemove.pop();
      nodes.remove(mid);
    }
  }

  /**
   * Prepares all events to simulate from a events vector
   * @param events Vector of events   
   */ 
  public void parseEvents(Vector events) throws InitializationException {
    IEvent aEvent;

    Iterator it = events.iterator();
    while (it.hasNext()){
      aEvent = (IEvent) it.next();
      switch (aEvent.getType()){
        case Globals.JOIN:		parseJoin(aEvent);
          break;
        case Globals.LEAVE:		parseLeave(aEvent);
          break;
        case Globals.LOOKUP:	parseLookup(aEvent);
          break;
        case Globals.FAIL: 		parseFail(aEvent);
          break;

      }
    }
  }

  /**
   * Prepares the join event to simulate from a event interface
   * @param aEvent IEvent
   * @see planet.simulate.IEvent  
   */ 
  public void parseJoin (IEvent aEvent) throws InitializationException {
    Id id, target;

    id = aEvent.getFrom();
    target = aEvent.getTo();
    
    if (id==null || target==null)
        throw new InitializationException("Cannot obtain the source or bootstrap node Id from the event.");

    if (nodes.containsKey(id)) {
      Logger.log("Node id ["+id+"] already exists on the network.",Logger.ERROR_LOG);
    } 
    else {
      addNode(id,target);   
    }
  }  
  
  /**
   * Prepares the leave event to simulate from a event interface
   * @param aEvent IEvent
   * @see planet.simulate.IEvent  
   */
  public void parseLeave(IEvent aEvent) throws InitializationException{
    Id id = aEvent.getFrom();
    if (id == null) {
      throw new InitializationException ("Cannot get the node Id to leave.");
    }

    Node aNode = (Node)nodes.get(id);
    aNode.leave();
    toRemove.push(id);
  }

  /**
   * Prepares the lookup event to simulate from a event interface
   * @param aEvent IEvent
   * @see planet.simulate.IEvent  
   */
  public void parseLookup(IEvent aEvent) throws InitializationException {
    Id id, target;

    id = aEvent.getFrom();
    target = aEvent.getTo();

    if (id== null || target == null) 
      throw new InitializationException("Cannot obtain the source or target node id in the lookup event.");
    
    
    Node aNode = (Node)nodes.get(id);
    ((NodeImpl)aNode).lookup(target);
  }

  /**
   * Prepares the fail event to simulate from a event interface
   * @param aEvent IEvent
   * @see planet.simulate.IEvent  
   */
  public void parseFail(IEvent aEvent) throws InitializationException {
    Id id;
    id = aEvent.getFrom();
    if (id== null) 
        throw new InitializationException("Cannot obtain the node id to fail.");

    nodes.remove(id);
  }

  /**
   * do nothing.
   * @see planet.simulate.ISimulator#stop()
   */
  public void stop() { }

  /**
   * Takes all messages of queue and send to all destination nodes
   * @param messages Queue with messages to send
   * @see planet.util.Queue  
   */
  public void send (Queue messages) {
    RouteMessage aMessage;
    NodeHandle from, target;
    Node aNode;
    int processed = 0;
    while (!messages.isEmpty()&& processed < Properties.msgProcess){
      aMessage = (RouteMessage)messages.remove();
      target = aMessage.getNextHopHandle();
      if (nodes.containsKey(target)) {
        aNode = (Node)nodes.get(target);

        try {
          aNode.receive(aMessage);
	    } catch (QueueFull ex){
          Logger.log("Incoming queue of Node "+target+" is full",Logger.EVENT_LOG);
          MessagePool.freeMessage(aMessage);
        }
      } else {
      	Logger.log("Target of message ["+aMessage+"] is not found.",Logger.MSG_LOG);
        from = aMessage.getSource();
        if (nodes.containsKey(from)){
          aNode = (Node)nodes.get(from);
          aMessage.setMode(Globals.ERROR);
          try {
          	NodeHandle source = aMessage.getSource();
          	aMessage.setSource(aMessage.getDestination());
          	aMessage.setDestination(source);
          	aMessage.setNextHopHandle(source);
            aNode.receive(aMessage);
          } catch (QueueFull ex) {
            Logger.log("Incoming queue of Node "+from+" is full",Logger.EVENT_LOG);
            MessagePool.freeMessage(aMessage);
          }
        }
      }

      processed ++;
    }
  }
  
  /**
   * Selects a node by a random number
   * @param r Random number
   * @return The chosen Node
   */
  public Node getRandomNode(Random r) {
	Collection values = nodes.values();
	int x = r.nextInt(values.size());
	Iterator it = values.iterator();
	int i = -1;
	Node n = null;
	while (i < x) {
	  n   = (Node)it.next();
	  i++;
	}
	return n;
  }
  
  /**
   * Return the size of the network
   * @return size int of the number of nodes
   */
  public int getSizeRing() {    
	return nodes.size();
  }
  
  /**
   * Returns the HashMap structure of the all nodes
   * @return A HashMap with all nodes mapped by their Id
   */
  public HashMap getNodes() {    
	return nodes;
  }
}
