package planet.symphony;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import planet.commonapi.Id;
import planet.commonapi.exception.InitializationException;
import planet.util.Utilities;


/**
 * This Id is backed up a double value. This value always must to be
 * between 0.0 and 1.0, as Symphony establish.
 * 
 * @author <a href="mailto: heliodoro.tejedor@estudiants.urv.es">Helio Tejedor</a>
 * @author <a href="mailto: jordi.pujol@estudiants.urv.es">Jordi Pujol</a>
 * @author <a href="mailto: marc.sanchez@estudiants.urv.es">Marc Sanchez</a>
 */
public class SymphonyId extends Id implements java.io.Serializable {

	/* ********************** SYMPHONY SPECIFIC CONSTANTS  ***************/
	/**
	 * Symphony specific constant: Maximum value for the internal value of Id.
	 */
	public static final double MAX_SYMPHONY = 1.0;
	
	
	/* ********************** SYMPHONY SPECIFIC CONSTANTS  ***************/
	
	
	/* *********** STATIC ATTRIBUTES FOR getMaximum() METHOD ********/
	/** Inform the maximum value for this SymphonyId (1.0). */
	protected static Id MAX;
	/* END *********** STATIC ATTRIBUTES FOR getMaximum() METHOD ********/
	
	/**
	 * Internal value for this Id.
	 */
	private double id;
	
	
	
	/**
	 * Must be initialized to initial value, in this case to [0.00].
	 */
	public SymphonyId() {
		id = 0.0;
	}
	
	/**
	 * Always throws an InitializationException, because this constructor
	 * is nonapplicable to this type of Id.
	 * @param id internal value as int argument.
	 * @throws InitializationException Always
	 */
	public SymphonyId(int id) throws InitializationException {
		throw new InitializationException("ERROR: This constructor is nonapplicable");
	}
	
	/**
	 * Builds a new SymphonyId with the decimals that appear in
	 *  <b>id</b> double argument.
	 * @param id double with the internal value to set to new Id.
	 */
	public SymphonyId(double id) {
		this.id = getDecimals(id);
	}

	/**
	 * Builds an Id with the data in byte[] format. This data must have
	 * to be builded as on SymphonyId.toByteArray(). 
	 *  
	 * @param data A byte[] with the internal value. 
	 * @see #toByteArray()
	 * @throws InitializationException if an error occurs during 
	 * the initialization. 
	 */
	public SymphonyId(byte[] data) throws InitializationException {
		long ba = 0;
		for(int i = data.length-1; i >=0; i--) {
			ba <<= 8;
			ba = ba | data[i];
		}
		id = Double.longBitsToDouble(ba);
	}
	
	/**
	 * Always throws an InitializationException because this constructor
	 * is nonapplicable to this type of Id.
	 * @param data int[] with the internal value to set to new Id.
	 * @throws InitializationException Always 
	 */
	public SymphonyId(int[] data) throws InitializationException {
		throw new InitializationException("ERROR: This constructor is nonapplicable");
	}
	
	/**
	 * Builds a new Id with the double value that appear in the argument
	 * as String value.
	 * @param sid String representation of a double value.
	 * @throws InitializationException if an error occurs during the
	 * initialization.
	 */
	public SymphonyId(String sid) throws InitializationException {
		try {
			id = Double.parseDouble(sid);
			id = getDecimals(id);
		} catch(NumberFormatException nfe) {
			throw new InitializationException("SymphonyId(String) NumberFormatException", nfe);
		}
	}
	
	/**
	 * Builds a new Id from an arbitray string applying a one-way hashing algorithm
	 * such as SHA or MD5.
	 * @param sid An arbitray string. 
	 * @param algorithm one-way hashing algorithm such as "SHA" or "MD5".
	 */
	public SymphonyId(String sid, String algorithm)throws InitializationException {
	    MessageDigest md = null;
		byte[] digest = null;
		BigInteger Mask = BigInteger.ONE;
		if (sid == null) throw new InitializationException("The arbitrary string is set to null"); 
		try {
		    md = MessageDigest.getInstance(algorithm);
			md.update(sid.getBytes());
			digest = md.digest();
		} catch (NoSuchAlgorithmException e) {
			throw new InitializationException("Not algorithm support!", e);
		}			
		Mask = Mask.shiftLeft(digest.length * Utilities.BYTE_SIZE);
		
		//always these are integer values ==> show them as double values to get double result
		BigDecimal wrapperDigest = new BigDecimal(new BigInteger(digest).toString()+".0");
		wrapperDigest = wrapperDigest.divide(new BigDecimal(Mask.toString()+".0"),0);
		  
		double id = wrapperDigest.doubleValue();
		id = getDecimals(id);
	}		 
	
	/**
	 * Always throws an InitializationException because this constructor
	 * is nonapplicable to this type of Id.
	 * @param value BigInteger with the internal value to the new Id.
	 * @throws InitializationException Always.
	 */
	public SymphonyId(BigInteger value) throws InitializationException {
		throw new InitializationException("ERROR: This constructor is nonapplicable");
	}
	
	/**
	 * Build a new random Id, based with the random number generator <b>random</b>.
	 * @param random A java.util.Random instance.
	 * @throws InitializationException if any error has occurred during the initialization.
	 */
	public SymphonyId(java.util.Random random) throws InitializationException {
	    this(random.nextDouble());
	}
	
	/**
	 * Gives back the value between 0.0 and 1.0 of the <b>value</b>,
	 * using only the decimals of the argument.
	 * @param value Double which gives back its decimals.
	 * @return The decimals that appear in the argument.
	 */
  	protected static double getDecimals(double value) {
		int i = (int) value;
		double ret = value - (double) i;
		if (ret < 0.0)
			ret += 1.0;
		return ret;
	}
	
	public Object getValue() {
		return new Double(id);
	}
	
	public double getDoubleValue() {
		return id;
	}
	
	public void setValue(double id) {
		this.id = id;
	}
	
// Id

	public boolean between(Id ccw, Id cw) {
		int comp = ccw.compareTo(cw);
		
		if (comp == 0)
			return true;
		else if (comp < 0)
			return ( (compareTo(ccw) > 0) && (compareTo(cw) < 0) );
		else
			return ( (compareTo(ccw) > 0) || (compareTo(cw) < 0) );
	}

	public boolean clockwise(Id nid) {
		/* Difference between your id and mine must be less than or equal 0.5 */
		double other = getDecimals( ((SymphonyId) nid).id - id );
		return (other <= 0.5);
	}
	
	public Id add(Id offset) {
		return new SymphonyId(id + ((SymphonyId) offset).id);
	}
	
	public Id subtract(Id offset) {
		return new SymphonyId(id - ((SymphonyId) offset).id);
	}
	
	/* fill not used! */
	public Id shift(int cnt, int fill) {
		double m = (double) Math.pow(2, cnt);
		double nid = this.id * m;
		return new SymphonyId(nid);
	}

	public byte[] toByteArray() {
		long ba = Double.doubleToRawLongBits(id);
		byte[] r = new byte[8];
		for(int i = 0; i < r.length; i++) {
			r[i] = (byte) (ba & 0xFF);
			ba >>= 8;
		}
		return r;
	}

	public String toStringFull() {
		return Double.toString(id);
	}
	
// Comparable
	public boolean equals(Object o) {
		return compareTo(o) == 0;
	}
	
	public int compareTo(Object o) {
		return Double.compare(id, ((SymphonyId) o).id );
	}

	public int hashCode() {
		long lb = Double.doubleToRawLongBits(id);
		return (int) (lb & 0xFFFF) ^ (int) (lb >> 8);
	}
	
	public String toString() {
		return Double.toString(id);
	}
	
	/**
	 * Sets the new value for this SymphonyId. This method
	 * waits an Double object as parameter.
	 * @see planet.commonapi.Id#setValue(java.lang.Object)
	 * @param newValue New double value for this Id, wrapped
	 * by a Double.
	 */
	public void setValue(Object newValue) {
		this.id = ((Double)newValue).doubleValue();
	}
	
	/**
	 * This method returns the maximum value for a SymphonyId that is 
     * possible to build. To return this maximum value use the
     * design pattern Singleton. 
     * @return The maximum value sets to an Id (1.0).
	 * @see planet.commonapi.Id#getMaximum()
	 */
	public static Id getMaximum() {
		if (MAX == null ) {
			MAX = new SymphonyId(MAX_SYMPHONY); 
		}
		return MAX;
	}
	
	/**
	 * Overwrite the static method to return the required Id.
	 * @param num Total number of nodes in the network.
	 * @return A new instance of this Id, with the increment between
	 * two consecutive Ids.
	 * @see planet.commonapi.Id#divide(long)
	 */
	public static Id divide(long num)
	{
	    return new SymphonyId((MAX_SYMPHONY/(double)num));
	}
	
}
