package planet.symphony.behaviours;

import java.util.Set;

import planet.behaviour.Behaviour;
import planet.commonapi.Node;
import planet.commonapi.NodeHandle;
import planet.commonapi.RouteMessage;
import planet.symphony.SymphonyNode;

/**
 * CloseLongConnectBehaviour attemps to handle a CLOSE_LONG_CONNECT request. Two cases are possible:
 * 		~ Node is stable: it has lost a long link and must send a new QUERY_CONNECT message in order
 *                        to get K links.
 *      ~ Node is not stable: it is already trying to get K long links, so not a new QUERY_CONNECT
 *                        message is necessary.
 * @author Marc Sanchez <marc.sanchez@estudiants.urv.es>
 * @author Helio Tejedor <heliodoro.tejedor@estudiants.urv.es> 
 */
public class CloseLongConnectBehaviour implements Behaviour  {
	/**
	 *  Given a RouteMessage and a Node as input, onMessage's method do execution 
	 * steps for the behaviour. Node should be casted to Node's class in order to
	 * manage data structures and protocol of designed overlay.
	 * @param msg RouteMessage taken as input.
	 * @param node Node taken as input.
	 * @return Returns either an array of RouteMessages or null when no messages
	 * need to transmit this node. 
	 */
	public RouteMessage[] onMessage(RouteMessage msg, Node node) {
		SymphonyNode symphony = (SymphonyNode) node;
		
		NodeHandle sourceHandle = msg.getSource();
		
		Set outcommingSet = symphony.getOutcommingSet();
		Set incommingSet  = symphony.getIncommingSet();
		Set newSet = symphony.getNewSet();
		
		newSet.remove(sourceHandle);
		incommingSet.remove(sourceHandle);
		boolean outRemoved = outcommingSet.remove(sourceHandle);
		if (outRemoved) {
			// Node is stable and has lost a long link. Therefore, it should send a new QUERY_CONNECT.
			if (symphony.stabilized) {
				RouteMessage newQueryConnect = symphony.getNewLongDistance();
				if (newQueryConnect == null) return null;
				return new RouteMessage[] { newQueryConnect };
			// Is not stable and thereby the symphony node's is already attempting to get K long lings.
			} else 	newSet.remove(sourceHandle);
		}
		return null;
	}
	/**
	 * @return Returns the name of behaviour. 
	 */
	public String getName() {
		return "CloseLongConnectBehaviour";
	}
	/**
	 * @return Returns a string representation of the behaviour. In general, the toString
	 * method returns a string that "textually represents" this behaviour. The result should 
	 * be a concise but informative representation that is easy for a person to read. 
	 */ 
	public String toString() {
		return getName();
	}
}
